#include "stdio.h"
#include "string.h"

#ifndef _BASE64_HEADER_
#define _BASE64_HEADER_

// Decode Base64 encoded src into dest, returns the number of data bytes
// stored in dest, or -1 on error
int DecodeBase64(char *src, char *dest, int szdest);

// Encode src in Base64 into dest, returns the number of data bytes
// stored in dest, or -1 on error
int EncodeBase64(char *src, char *dest, int szdest);


static const char Pad64 = '=';
static const char Base64[] =
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";


int DecodeBase64(char *src, char *dest, int szdest)
{
	int destidx, state, ch;
	char *pos;

	state = 0;
	destidx = 0;

	while ((ch = *src++) != '\0')
	{
		if (isspace(ch)) // Skip whitespace
			continue;

		if (ch == Pad64)
			break;

		pos = strchr(Base64, ch);
		if (pos == 0) // A non-base64 character
			continue;

		switch (state) {
		case 0:
			if (dest)
			{
				if (destidx >= szdest)
					return (-1);
				dest[destidx] = (pos - Base64) << 2;
			}
			state = 1;
			break;
		case 1:
			if (dest)
			{
				if (destidx + 1 >= szdest)
					return (-1);
				dest[destidx]   |=  (pos - Base64) >> 4;
				dest[destidx+1]  = ((pos - Base64) & 0x0f) << 4 ;
			}
			destidx++;
			state = 2;
			break;
		case 2:
			if (dest)
			{
				if (destidx + 1 >= szdest)
					return (-2);
				dest[destidx]   |=  (pos - Base64) >> 2;
				dest[destidx+1]  = ((pos - Base64) & 0x03) << 6;
			}
			destidx++;
			state = 3;
			break;
		case 3:
			if (dest)
			{
				if (destidx >= szdest)
					return (-3);
				dest[destidx] |= (pos - Base64);
			}
			destidx++;
			state = 0;
			break;
		}
	}

	// We are done decoding Base-64 chars.  Let's see if we ended
	//	on a byte boundary, and/or with erroneous trailing characters.

	if (ch == Pad64)		// We got a pad char
	{
		ch = *src++;		// Skip it, get next
		switch (state) {
		case 0:		// Invalid = in first position
		case 1:		// Invalid = in second position
			return (-4);

		case 2:		// Valid, means one byte of info
			// Skip any number of spaces
			for (NULL; ch != '\0'; ch = *src++)
				if (!isspace(ch))
					break;
			// Make sure there is another trailing = sign
			if (ch != Pad64)
				return (-5);
			ch = *src++;		// Skip the =
			// Fall through to "single trailing =" case
			// FALLTHROUGH

		case 3:		// Valid, means two bytes of info
			// We know this char is an =.  Is there anything but
			//	whitespace after it?
			for (NULL; ch != '\0'; ch = *src++)
				if (!isspace(ch))
					return (-6);

			// Now make sure for cases 2 and 3 that the "extra"
			//	bits that slopped past the last full byte were
			//	zeros.  If we don't check them, they become a
			//	subliminal channel.
			if (dest && dest[destidx] != 0)
				return (-7);
		}
	}
	else
	{
		// We ended by seeing the end of the string.  Make sure we
		//	have no partial bytes lying around.
		if (state != 0)
			return (-8);
	}

	return (destidx);
}

int EncodeBase64(char *src, char *dest, int szdest)
{
	char a, b, c, d, *cp, *dst;
	int i, srclen, enclen, remlen;

	cp = src;
	dst = dest;
	srclen = strlen(cp);			// length of source
    enclen = srclen / 3;			// number of 4 byte encodings (source DIV 3)
	remlen = srclen - 3 * enclen;	// remainder if srclen not divisible by 3 (source MOD 3)
    
    for (i = 0; i < enclen; i++)
	{
		a = (cp[0] >> 2);
        b = (cp[0] << 4) & 0x30;
        b |= (cp[1] >> 4);
        c = (cp[1] << 2) & 0x3c;
        c |= (cp[2] >> 6);
        d = cp[2] & 0x3f;
        cp +=3;
		if (dst + 6 - dest > szdest) return -1;
        sprintf(dst, "%c%c%c%c",Base64[a],Base64[b],Base64[c],Base64[d]);
		dst+=4;
	}

	if (remlen == 0)
		return (dst - dest + 1);

	if (remlen == 1)
	{
		a = (cp[0] >> 2);
        b = (cp[0] << 4) & 0x30;
        if (dst + 6 - dest > szdest) return -2;
        sprintf(dst, "%c%c==",Base64[a],Base64[b]);
		dst+=4;
	}
	else // remlen == 2
	{
		a = (cp[0] >> 2);
        b = (cp[0] << 4) & 0x30 ;
        b |= (cp[1] >> 4);
        c = (cp[1] << 2) & 0x3c;
        if (dst + 6 - dest > szdest) return -3;
        sprintf(dst, "%c%c%c=",Base64[a],Base64[b],Base64[c]);
		dst+=4;
	}
	return dst - dest + 1;
}

#endif // _BASE64_HEADER_